import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import QtMultimedia

import Ad
import Core
import I18n

import "../core"
import "../widgets"

BaseForm {
    id: root
    Style.class: "adDisplayForm"

    Context.context {
        inputSources: InputSources.All
        fullScreen: true
    }

    property var adItem: Ad.currentItem
    property bool displayed: Ad.displayed
    onDisplayedChanged: if (!displayed) nextTimer.stop()
    onAdItemChanged: {
        if (adItem.time !== 0)
            nextTimer.start()
        if (player.source == adItem.url)
            player.source = ""
        player.source = ImageData.isImage(adItem.url) ? "" : adItem.url
    }

    MediaPlayer {
        id: player
        source: !ImageData.isImage(adItem.url) ? adItem.url : ""
        loops: adItem.time > 0 && Ad.itemsForShow.length > 1 ? MediaPlayer.Infinite : 1
        onMediaStatusChanged: {
            if (player.mediaStatus === MediaPlayer.BufferedMedia) {
                play()
            } else if (player.mediaStatus === MediaPlayer.EndOfMedia) {
                player.source = ""
                PluginManager.internalAsync("AD_NEXT")
            }
        }
        onErrorOccurred: Ad.currentItemError()
        videoOutput: videoOutput
        onSourceChanged: play()

        Component.onCompleted: play()
    }

    ColumnLayout {
        anchors.fill: parent
        Layout.fillWidth: true
        Layout.fillHeight: true

        SGroupBox {
            Style.class: "adDisplayFormGroupBox"
            Layout.fillHeight: true
            Layout.fillWidth: true

            VideoOutput {
                id: videoOutput
                anchors.fill: parent
                visible: !ImageData.isImage(adItem.url)
            }

            AnimatedImage {
                anchors.fill: parent
                visible: ImageData.isImage(adItem.url)
                source: ImageData.isImage(adItem.url) ? adItem.url : ""
                fillMode: Image.PreserveAspectFit
                onStatusChanged: {
                    switch (status) {
                    case AnimatedImage.Ready:
                        playing = true
                        break
                    case AnimatedImage.Error:
                        Ad.currentItemError()
                    }
                }
            }

            MouseArea {
                z: 1
                anchors.fill: parent
                onClicked: if (!Ad.secondScreen) adStopAction.exec()
            }

            ActionButton {
                z: 3
                Style.class: "adDisplayFormClose"
                visible: Config.getBool("Ad.DisplayForm.stopButton:visible", true)
                anchors {
                    bottom: parent.bottom
                    right: parent.right
                    left: parent.left

                    leftMargin: Style.layout.leftPadding
                    rightMargin: Style.layout.rightPadding
                    bottomMargin: Style.layout.bottomPadding
                }

                Tr { code: "adStop" }
                onClicked: adStopAction.exec()

                Layout.fillWidth: true
                implicitHeight: Style.content.height
            }
        }
    }

    Action {
        id: adStopAction
        command: "AD_STOP"
    }

    Timer {
        id: nextTimer
        repeat: false
        interval: adItem.time * 1000
        onTriggered: PluginManager.internalAsync("AD_NEXT")
    }
}
