import QtQuick
import QtQuick.Controls
import QtQuick.Layouts

import Check
import Core
import I18n

import "../core"
import "../widgets"

BaseForm {
    id: root
    Style.class: "discInfoForm"

    focusPolicy: Qt.ClickFocus

    ColumnLayout {
        anchors.fill: parent
        Layout.fillWidth: true
        spacing: 20

        SGroupBox {
            Layout.fillHeight: true
            Layout.fillWidth: true

            ColumnLayout {
                anchors.fill: parent

                ColumnLayout {
                    Layout.leftMargin: 90
                    Layout.rightMargin: Layout.leftMargin
                    Layout.topMargin: 55
                    Layout.bottomMargin: 35
                    Layout.fillWidth: true
                    Layout.fillHeight: true

                    SLabel {
                        Style.class: "discInfoTitle"
                        horizontalAlignment: Text.AlignHCenter

                        Tr { code: "discInfoTitle" }

                        Layout.preferredHeight: contentHeight
                        Layout.preferredWidth: contentWidth
                        Layout.alignment: Qt.AlignHCenter
                    }

                    Space { height: 30 }

                    LoyaltyButtons {
                        id: loyaltyInfo
                        inputCouponTrCode: "discInfoInputCoupon"
                        scanCardTrCode: "discInfoScanLoyaltyCard"
                        cardBalanceTrCode: "discInfoCardBalance"

                        inputCouponButton {
                            Style.class: "discInfoInputCoupon"
                            visible: false
                        }
                        animationStyleSection: ".discInfo"
                        inputCardButton.Style.class: "discInfoInputCard"
                        model.onLayoutChanged: () => discountLayout.update()
                    }

                    Space { height: 30 }

                    DashLineSeparator {
                        Layout.fillWidth: true; Layout.alignment: Qt.AlignTop;
                        dashPattern: [1, 0]; strokeWidth: 1; color: Style.color("graySoft")
                    }

                    SGroupBox {
                        id: mainBox
                        leftPadding: 0
                        rightPadding: leftPadding
                        topPadding: 30
                        bottomPadding: 0
                        Layout.fillWidth: true
                        Layout.fillHeight: true

                        contentItem: Item {
                            anchors.fill: parent

                            ColumnLayout {
                                anchors.fill: parent
                                spacing: 10

                                PositionList {
                                    id: positionList
                                    Layout.fillHeight: true
                                    Layout.fillWidth: true

                                    CheckModel { id: checkModel }
                                    model: FilterModel {
                                        id: model
                                        sourceModel: checkModel
                                        filterFunction: (pos) => pos.type == CheckTypes.ItemType.Position || pos.type == CheckTypes.ItemType.Coupon
                                    }
                                    backgroundColor: mainBox.background.color

                                    property real delegateHeight: 100
                                    property real delegatePositionSumWidth: 50

                                    delegate: ColumnLayout {
                                        width: positionList.width
                                        height: positionList.delegateHeight

                                        RowLayout {
                                            Layout.fillWidth: true; Layout.alignment: Qt.AlignVCenter
                                            Space { width: 10 }
                                            SLabel {
                                                Layout.fillWidth: true
                                                Style.class: "discInfoPositionName"
                                                text: model.name

                                                Tr {
                                                    code: { return "couponName" }
                                                    args: [name]
                                                    enabled: type === CheckTypes.ItemType.Coupon
                                                }

                                                elide: Text.ElideRight
                                                wrapMode: Text.WordWrap
                                                maximumLineCount: 2
                                            }
                                            Space { width: 20 }
                                            SLabel {
                                                leftPadding: 17; rightPadding: 17; topPadding: 6; bottomPadding: 6
                                                visible: model.type === CheckTypes.ItemType.Position
                                                Style.class: "discInfoPositionQuantity"
                                                text: model.quantity ? "x" + model.quantity.str : ""
                                            }
                                            Space { width: 20 }
                                            Control {
                                                visible: model.type === CheckTypes.ItemType.Position
                                                implicitWidth: {
                                                    return positionList.delegatePositionSumWidth = Math.max(
                                                            positionList.delegatePositionSumWidth, positionSum.width)
                                                }

                                                contentItem: ColumnLayout {
                                                    anchors.fill: parent
                                                    spacing: 5
                                                    Money {
                                                        id: positionSum
                                                        Style.class: "discInfoPositionSumNoDisc"
                                                        money: model.sumNoDisc
                                                        color: styleColor
                                                        backgroundColor: Style.color("background")
                                                        Layout.alignment: Qt.AlignRight
                                                    }
                                                    Money {
                                                        Style.class: "discInfoPositionDiscSum"
                                                        prefix: "-"
                                                        money: (model.disc)
                                                        visible: model.disc ? !model.disc.zero : false
                                                        color: styleColor
                                                        backgroundColor: Style.color("background")
                                                        Layout.alignment: Qt.AlignRight
                                                    }
                                                }
                                            }
                                            Space { width: 30 }
                                        }
                                        DashLineSeparator {
                                            Layout.fillWidth: true; Layout.alignment: Qt.AlignBottom
                                            dashPattern: [1, 0]; strokeWidth: 1; color: Style.color("graySoft")
                                        }
                                    }
                                }

                                DashLineSeparator {
                                    Layout.fillWidth: true; Layout.alignment: Qt.AlignBottom
                                    dashPattern: [1, 0]; strokeWidth: 1; color: Style.color("graySoft")
                                }

                                ColumnLayout {
                                    id: discountLayout
                                    property ListModel discInfo: ListModel { id: discountInfoModel }

                                    FilterModel {
                                        id: cards
                                        sourceModel: CheckModel { }
                                        filterFunction: (pos) => pos.type == CheckTypes.ItemType.Card && pos.hasBalance

                                        onLayoutChanged: () => discountLayout.update()
                                    }

                                    Layout.fillWidth: true
                                    Component.onCompleted: () => update()
                                    function update() {
                                        discountInfoModel.clear()
                                        const cardIndex = cards.index(0, 0)
                                        if (cards.count > 0 && cardIndex.valid) {
                                            discInfo.append({ labelTr: "discInfoBalance", labelStyle: "discInfoBalance",
                                                              value: cards.data(cardIndex, CheckModel.Balance), valueStyle: "discInfoBalanceValue"})
                                            discInfo.append({ labelTr: "discInfoEarn", labelStyle: "discInfoEarn",
                                                              value: cards.data(cardIndex, CheckModel.Earn), valueStyle: "discInfoEarnValue"})
                                            discInfo.append({ labelTr: "discInfoSpend", labelStyle: "discInfoSpend",
                                                              value: cards.data(cardIndex, CheckModel.CanSpend), valueStyle: "discInfoSpendValue"})
                                        }
                                        discInfo.append({ labelTr: "discInfoDisc", labelStyle: "discInfoDiscountSum",
                                                          value: Check.discount, valueStyle: "discInfoDiscountSumValue" })
                                    }

                                    Repeater {
                                        id: discountInfo
                                        model: discountInfoModel
                                        delegate: RowLayout {
                                            spacing: 40

                                            SLabel {
                                                Style.class: model.labelStyle
                                                Tr { code: model.labelTr }
                                                Layout.fillWidth: true
                                            }
                                            Money {
                                                Style.class: model.valueStyle
                                                money: model.value
                                                color: styleColor
                                                backgroundColor: Style.color("background")
                                            }

                                            Layout.fillWidth: true
                                            Layout.preferredHeight: 60
                                            width: discountInfo.width
                                            Layout.leftMargin: 20
                                        }
                                    }
                                }

                                DashLineSeparator {
                                    Layout.fillWidth: true; Layout.alignment: Qt.AlignBottom
                                    dashPattern: [1, 0]; strokeWidth: 1; color: Style.color("graySoft")
                                }

                                RowLayout {
                                    spacing: 40

                                    SLabel {
                                        Style.class: "discInfoDue"
                                        Tr { code: "discInfoDue" }
                                        Layout.fillWidth: true
                                    }
                                    Money {
                                        Style.class: "discInfoDueValue"
                                        money: Check.total
                                        color: styleColor
                                        backgroundColor: Style.color("background")
                                    }

                                    Layout.fillWidth: true
                                    Layout.preferredHeight: 60
                                    Layout.leftMargin: 20
                                    Layout.bottomMargin: 20
                                }
                            }
                        }
                    }
                }
            }
        }

        ActionButton {
            id: continueButton
            Style.class: "discInfoClose"

            Tr { code: "discInfoClose" }
            onClicked: () => removeContext()

            Layout.fillWidth: true
            Layout.preferredHeight: 110
            Layout.topMargin: 20
        }
    }
}
